// Title:   ActionManager.cpp
// Created: Thu Sep 25 15:17:00 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include "ActionManager.h"
#include <qapplication.h>

#ifdef _WIN32
	// turn off std::map warnings
	//# pragma warning(push)
	# pragma warning(disable:4786) // identifier truncated to 255 char
#endif

#include "NewOverload.h"

ActionManager::ActionManager(unsigned int max_stack_size) { 
	_max_stack_size = max_stack_size;
	clear();
}

void ActionManager::clear() {
	_undo_stack.clear();
	_redo_stack.clear();
}

void ActionManager::undo() {
	QApplication::setOverrideCursor(Qt::waitCursor);

	UndoAction *redo;

	if (_undo_stack.size() == 0) {
		QApplication::restoreOverrideCursor();
		return;
	}

	redo = _undo_stack.back()->undo();
	_redo_stack.push_back(redo);
	
	if (_redo_stack.size() > _max_stack_size) {
		_MCdelete((unsigned long)_redo_stack.front());
		delete _redo_stack.front();
		_redo_stack.pop_front();
	}

	_MCdelete((unsigned long)_undo_stack.back());
	delete _undo_stack.back();

	_undo_stack.pop_back();

	QApplication::restoreOverrideCursor();
}

void ActionManager::redo() {
	QApplication::setOverrideCursor(Qt::waitCursor);
	
	UndoAction *undo;

	if (_redo_stack.size() == 0){
		QApplication::restoreOverrideCursor();
		return;
	}

	undo = _redo_stack.back()->undo();
	_undo_stack.push_back(undo);

	if (_undo_stack.size() > _max_stack_size) {
		_MCdelete((unsigned long)_undo_stack.front());
		delete _undo_stack.front();
		_undo_stack.pop_front();
	}

	_MCdelete((unsigned long)_redo_stack.back());
	delete _redo_stack.back();
	
	_redo_stack.pop_back();
	
	QApplication::restoreOverrideCursor();
}

void ActionManager::addUndoAction(UndoAction *undo) {
	if (undo == 0)
		return;

	
	// clear redo stack
	while (_redo_stack.size() > 0) {
		_MCdelete((unsigned long)_redo_stack.front());
		delete _redo_stack.front();
		_redo_stack.pop_front();
	}
	
	_undo_stack.push_back(undo);
	if (_undo_stack.size() > _max_stack_size) {
		_MCdelete((unsigned long)_undo_stack.front());
		delete _undo_stack.front();
		_undo_stack.pop_front();
	}
}
// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
