// Title:   ActionManager.h
// Created: Thu Sep 25 15:16:58 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __ACTIONMANAGER_H
#define __ACTIONMANAGER_H

#ifdef _WIN32
	// turn off std::map warnings
	# pragma warning(push)
	# pragma warning(disable:4786) // identifier truncated to 255 char
#endif

#include <limits.h>
#include <deque>
#include "Action.h"

/**
 * Action Manager
 * manages the stack of Undo/Redo actions.
 *
 * @author Simon Heinzle
 * @version 1.0
 */
class ActionManager {
	unsigned int _max_stack_size;
	std::deque<UndoAction*> _undo_stack;
	std::deque<UndoAction*> _redo_stack;
public:
	/**
	 * Constructor
	 *
	 * Parameter <code>unsigned int max_stack_size</code> defines the maximum length of the undo and
	 * redo stack.
	 */
	ActionManager(unsigned int max_stack_size = UINT_MAX);
	/**
	 * Clears both undo and redo stack.
	 */
	void clear();
	/**
	 * Undoes the top undo operation of the stack and inserts the inverse operation in the redo stack.
	 */
	void undo();
	/**
	 * Redoes the top redo operation of the stack and inserts the inverse operation in the undo stack.
	 */
	void redo();
	/**
	 * Inserts the <code>UndoAction *undo</code> on top of the undo stack, clears the redo stack.
	 */
	void addUndoAction(UndoAction *undo);
};


#ifdef _WIN32
	#pragma warning(pop)
#endif // _WIN32

#endif
// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
