// Title:   MLSScatterer.h
// Created: Thu Sep 25 15:16:36 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __MLSSCATTERER_H
#define __MLSSCATTERER_H

#ifdef _WIN32
	// turn off std::map warnings
	# pragma warning(push)
	# pragma warning(disable:4786) 
#endif

#include "Action.h"
#include "BoxSelector.h"
#include "PlaneFit.h"
#include <vector>

#include "../../../../Core/Scene/src/Scene.h"
#include "../../../../Core/DataStructures/src/PriorityQueue.h"
#include "../../../../Core/DataStructures/src/QueryGrid.h"

typedef unsigned int* uIntPtr;
typedef unsigned int  uInt;

class MLSScatterer : public NormalAction {
	PlaneFit _planefit;
	Vector3D _plane_projected_position;
	Vector3D _poly_projected_position;
	unsigned int _neighbourhood_size;
	unsigned int _no_of_points;
	
	float maxdiameter;
	std::vector<unsigned int> _selection_indices;
	std::vector<Vector3D> _new_positions;
	QueryGrid<uInt> *_new_query_grid;

	MinPriorityQueue<unsigned int, float> _distance_pqueue;
	
	// which method to insert points?
	Vector3D *_center;
	float *_radius;
	BoxSelector *_boxselector;

	// check against high density?
	bool _density_check;
	// mls setting
	bool _use_mls;
	float _param_mls;


	void doSorting();
	void planeProjector(const Vector3D &source_position, int max_iterations = 10, float min_error = 1e-3);
	void polynomialProjection(const Vector3D &source_position);
	
	void reColorSurfels(const std::vector<SurfelInterface*> &sfls,const std::vector<Vector3D> &poses);

public:
	MLSScatterer();
	~MLSScatterer();
	void getNewPositionsBox(BoxSelector *box_selector);
	void getNewPositionsAxisAlignedBox(Vector3D min, Vector3D max);
	void setParameters(unsigned int no_of_points,unsigned int neighbourhood_size);
	
	void setMLSGauss(float param);
	void useMLS(bool use_it_or_loose_it);
	void setDensityCheck(bool check_density);
	void execute();
	UndoAction *executeUndo();
};


#ifdef _WIN32
	#pragma warning(pop)
#endif // _WIN32

#endif
// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
