// Title:   OutlierClassificationDialogImpl.h
// Created: Thu Sep 25 15:16:15 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __OutlierClassificationDialogImpl_H_
#define __OutlierClassificationDialogImpl_H_


#include "../UI/OutlierClassificationWindow.h"

#include "OutlierClassificator.h"

/**
 * Dialog Class for OutlierClassification
 *
 * @author Simon Heinzle
 * @version 0.0
 */
class OutlierClassificationDialogImpl : public OutlierClassificationWindow {
	
    Q_OBJECT

public:

	/**
	 * This method creates an instance of this <code>OutlierClassificationDialogImpl</code> if necessary,
	 * and returns it. Only one instance at a time exists (singleton pattern).
	 *
	 * @return an instance of this <code>OutlierClassificationDialogImpl</code>
	 */
	static OutlierClassificationDialogImpl *getInstance (void);

	/**
	 * Destroys the instance of this <code>OutlierClassificationDialogImpl</code>, if there
	 * is one.
	 */
	static void destroyInstance();

	void passMousePress(QMouseEvent *me);
	void passMouseRelease(QMouseEvent *me);
	void passMouseMove(QMouseEvent *me);
	
	void passKeyPressEvent     (QKeyEvent   *ke);
	void passKeyReleaseEvent   (QKeyEvent   *ke);

	void setQCursor(QCursor *new_cursor);
	
	void allPreview();
	UndoAction* allDelete();
	void classify();
public slots:
	void handleBuildButton();
	void handleMBClassifyButton();
	void handlePFClassifyButton();
	void handleBIClassifyButton();
	void handleALLClassifyButton();

	void handleMBPreviewDeleteButton();
	void handleMBDeleteButton();
	
	void handlePFPreviewDeleteButton();
	void handlePFDeleteButton();

	void handleBIPreviewDeleteButton();
	void handleBIDeleteButton();

	void handleALLPreviewDeleteButton();
	void handleALLDeleteButton();

	void handleMBupperSlider();
	void handlePFupperSlider();
	void handleBIupperSlider();

	void handleMethod1Slider();
	void handleMethod2Slider();
	void handleMethod3Slider();

	void handleRestoreButton();

	void handleCombinerMethod1SliderMoved(int value);
	void handleCombinerMethod2SliderMoved(int value);
	void handleCombinerMethod3SliderMoved(int value);
private:
	// singleton pattern, private c'tor
	OutlierClassificationDialogImpl (QWidget* parent = 0, const char* name = 0, WFlags flags = 0);
	void disableButtonsAndBox();
	void initDialog();
	
	QCursor *_tool_cursor;
	
	// ocl properties
	OutlierClassificator ocl;
	// ocl weights
	float w1,w2,w3;
	bool moving;

protected:	// To suppress warnings...
	virtual ~OutlierClassificationDialogImpl();

};

#endif  // __OutlierClassificationDialogImpl_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
