// Title:   OutlierClassificator.h
// Created: Thu Sep 25 15:15:57 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __OUTLIERCLASSIFICATOR_H
#define __OUTLIERCLASSIFICATOR_H

#ifdef _WIN32
	// turn off std::map warnings
	//# pragma warning(push)
	# pragma warning(disable:4786 4275 4800)
#endif


#define DEFAULT_K 50
#define DEFAULT_DISCARD_RATIO 0.2f

#include "../../../../Core/Scene/src/Scene.h"
#include <vector>

#include <qdatetime.h>
#if QT_VERSION >= 0x030000
# include <qptrvector.h>
# ifndef QVector
#  define QVector QPtrVector
# endif
# ifndef QVectorIterator
#  define QVectorIterator QPtrVectorIterator
# endif
#else
# include <qvector.h>
#endif

#include "Action.h"

class OutlierClassificator {
	unsigned int _k;
	unsigned int _count;

	std::vector<unsigned int> *_indices;
	std::vector<unsigned int> *_kNext_graph;
	std::vector<unsigned int> *_kNext_graph_sorted;

	std::vector<SurfelInterface*> _surfels;
	std::vector<QRgb> _old_diffuse;
	std::vector<QRgb> _old_specular;

	int _selection_index;
	float _discard_ratio;

	float *_classification_mb;
	float _max_c_mb, _min_c_mb;
	
	float *_classification_pf;
	float *_confidence_pf;
	float _max_c_pf, _min_c_pf;
	float _max_conf_pf, _min_conf_pf;
	
	float *_classification_bi;
	float _max_c_bi, _min_c_bi;
	
	float _tmp_co_mb, _tmp_co_pf, _tmp_co_bi;
	
	QTime _time_overall;
	
	float getCriterion(float weight_mb, float cutoff_mb, 
					   float weight_pf, float cutoff_pf,
					   float weight_bi, float cutoff_bi,
					   int real_index);
	void getCutoffs(float percentage, float &cutoff_mb, float &cutoff_pf, float &cutoff_bi);

public:
	OutlierClassificator(int k = DEFAULT_K, float d = DEFAULT_DISCARD_RATIO);
	~OutlierClassificator();
	void setK(int k) { _k = k; }
	void setDiscardRatio(float d) { _discard_ratio = d; if (d>1) _discard_ratio = 1.f; else if (d<0) _discard_ratio = 0.f; }
	bool buildKNextGraph(int selection_index);
	bool doMiniballClassification(int selection_index);
	bool doPlaneClassification(int selection_index);
	bool doBidirectionClassification(int selection_index);
	bool scaleClassification();
	bool colorSurfelsMB(float lower, float upper);
	bool colorSurfelsMB();
	// following saves old colors
	bool colorSurfelsCutoffMB(float cutoff);
	
	UndoAction* deleteSurfelsCutoffMB(float cutoff);

	bool colorSurfelsPF(float lower, float upper);
	bool colorSurfelsPF();
	bool colorSurfelsConfidencePF(float lower, float upper);
	bool colorSurfelsConfidencePF();
	// following saves old colors
	bool colorSurfelsCutoffPF(float cutoff, float cutoff_confidence);
	bool colorSurfelsCutoffPF(float cutoff);
	UndoAction* deleteSurfelsCutoffPF(float cutoff, float cutoff_confidence);
	UndoAction* deleteSurfelsCutoffPF(float cutoff);
	
	bool colorSurfelsBI(float lower, float upper);
	bool colorSurfelsBI();
	// following saves old colors
	bool colorSurfelsCutoffBI(float cutoff);
	UndoAction* deleteSurfelsCutoffBI(float cutoff);

	// following saves old colors
	bool colorSurfelsCutoffALL(float weight_mb, float cutoff_mb, 
		                       float weight_pf, float cutoff_pf,
							   float weight_bi, float cutoff_bi);
	UndoAction* deleteSurfelsCutoffALL(float weight_mb, float cutoff_mb, 
		                        float weight_pf, float cutoff_pf,
							    float weight_bi, float cutoff_bi);
	
	bool colorSurfelsPercentALL(float percentage, float weight_mb, float weight_pf, float weight_bi);
	UndoAction* deleteSurfelsPercentALL(float percentage, float weight_mb, float weight_pf, float weight_bi);
	
	// restoreColor for colorSurfelsCutoffXX
	bool restoreColor();

	bool testColor(int selection_index, int mode, float r, float g, float b, float coeff);
	
	float mbMaxC();
	float mbMinC();
	float pfMaxC();
	float pfMinC();
	float pfMaxConf();
	float pfMinConf();
	float biMaxC();
	float biMinC();

	float getCurrentCutoffMB() { return _tmp_co_mb; }
	float getCurrentCutoffPF() { return _tmp_co_pf; }
	float getCurrentCutoffBI() { return _tmp_co_bi; }
};
/*
#ifdef _WIN32
	#pragma warning(pop)
#endif // _WIN32
*/
#endif

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
