// Title:   RandomNoise.cpp
// Created: Thu Sep 25 15:15:50 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include "RandomNoise.h"

#ifdef _WIN32
	// turn off std::map warnings
	//# pragma warning(push)
	# pragma warning(disable:4786) 
#endif

#include "UndoRedoSpecialisations.h"

#include <vector>
#include <stdlib.h>
#include <qdatetime.h>
#include <qapplication.h>

#include "NewOverload.h"

void RandomNoise::setParameters(float alpha) {
	_alpha = alpha;
}


void RandomNoise::execute() {
	QApplication::setOverrideCursor(Qt::waitCursor);

	Selector *selector;
	std::vector<unsigned int> *selection;
	Vector3D position;
	Vector3D new_position;
	Vector3D normal;

	float random;
	unsigned int i;
	unsigned int index;

	selector = Selector::getInstance();
	selection = selector->getSelectionIndicesStdVec(Selector::intToFlag(_selection_index));

	srand(QTime::currentTime().msec());

	for (i = 0; i<selection->size(); i++) {
		index = (*selection)[i];

		position = selector->getSurfelWorldPosition(index);
		selector->getSurfelWorldNormal(index,normal);

		random = (float) rand() /  (float) RAND_MAX;
//		random = random * 2 - 1;

		new_position = position + random * _alpha * normal;

		selector->setSurfelWorldPosition(index, new_position, false);
	}

	if (i!=0)
		selector->setSurfelWorldPosition(index, new_position, true);
	QApplication::restoreOverrideCursor();

	_MCdelete((unsigned long)selection);
	delete selection;
}

UndoAction* RandomNoise::executeUndo() {
	QApplication::setOverrideCursor(Qt::waitCursor);

	Selector *selector;
	std::vector<unsigned int> *selection;
	Vector3D position;
	Vector3D new_position;
	Vector3D normal;
	
	SurfelInterface *surfel;
	std::vector<SurfelInterface*> sfls;
	std::vector<Vector3D> poses;
	std::vector<Vector3D> normals;

	float random;
	unsigned int i;
	unsigned int index;

	selector = Selector::getInstance();
	selection = selector->getSelectionIndicesStdVec(Selector::intToFlag(_selection_index));

	srand(QTime::currentTime().msec());

	for (i =0; i<selection->size(); i++) {
		index = (*selection)[i];

		position = selector->getSurfelWorldPosition(index);
		selector->getSurfelWorldNormal(index,normal);

		random = (float) rand() /  (float) RAND_MAX;
		random = random * 2 - 1;

		new_position = position + random * _alpha * normal;

		surfel = selector->getSurfel(index);
		sfls.push_back(surfel);
		poses.push_back(surfel->getPosition());
		normals.push_back(surfel->getNormal());
		selector->setSurfelWorldPosition(index, new_position, false);
	}

	if (i!=0) 
		selector->setSurfelWorldPosition(index, new_position, true);
	
	QApplication::restoreOverrideCursor();
	
	UndoAction *undo = new PositionNormalUndo(sfls,poses,normals);
	_MCnew( (unsigned long)undo, sizeof(undo), __FILE__, __LINE__);
	
	_MCdelete((unsigned long)selection);
	delete selection;
	return undo ;
}
// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
