// Title:   ScanToolsBoxColorDialogImpl.cpp
// Created: Thu Sep 25 15:15:42 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qfiledialog.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#if QT_VERSION >= 0x030000
# include <qptrlist.h>
# ifndef QList
#  define QList QPtrList
# endif
# ifndef QListIterator
#  define QListIterator QPtrListIterator
# endif
#else
# include <qlist.h>
#endif
#include <qstring.h>
#include <qfile.h>
#include <qbuttongroup.h>
#include <qstringlist.h>
#include <qcolordialog.h>
#include <qslider.h>
#include <qapplication.h>

#if QT_VERSION >= 0x030000
# include <qptrvector.h>
# ifndef QVector
#  define QVector QPtrVector
# endif
# ifndef QVectorIterator
#  define QVectorIterator QPtrVectorIterator
# endif
#else
# include <qvector.h>
#endif

#include "../../../../DataTypes/src/Vector3D.h"
#include "../../../../Utilities/src/Matrix.h"
#include "../../../../Core/Scene/src/Scene.h"

#include "../UI/ScanToolsBoxColorWidget.h"
#include "ScanToolsBoxColorDialogImpl.h"

#include "NewOverload.h"
#include "ScanToolsConfigurationDialogImpl.h"


// ########################################
// singleton pattern, the singleton itself
// ########################################
static ScanToolsBoxColorDialogImpl *instance;
	

// #####################################
//  c'tor & d'tor (private / protected)
// #####################################

ScanToolsBoxColorDialogImpl::ScanToolsBoxColorDialogImpl (QWidget* parent, const char* name, WFlags flags) 
		: ScanToolsBoxColorWidget(parent, name, flags)
{
	the_box_ = NULL; 
	this->connect (this->ApplyButton, SIGNAL (released()),
		           this, SLOT (handleApplyButton()));
}

ScanToolsBoxColorDialogImpl::~ScanToolsBoxColorDialogImpl(void)
{

}

// ####################################
//  public methods
// ####################################

ScanToolsBoxColorDialogImpl *ScanToolsBoxColorDialogImpl::getInstance(void) {
	if (instance == 0) {
		instance = new ScanToolsBoxColorDialogImpl ();
		_MCnew( (unsigned long)instance, sizeof(instance), __FILE__, __LINE__);
	}
	
	
	return instance;
}

void ScanToolsBoxColorDialogImpl::destroyInstance(void) {
	if (instance) {
		instance->close();
		_MCdelete((unsigned int)instance);
		delete instance;
	}
	instance = 0;
}

void ScanToolsBoxColorDialogImpl::initDialog(void) {
}



// ####################################
//  public slots
// ####################################




void ScanToolsBoxColorDialogImpl::passMousePress(QMouseEvent *me) { 
} 

void ScanToolsBoxColorDialogImpl::passMouseRelease(QMouseEvent *me){ 
}

void ScanToolsBoxColorDialogImpl::passMouseMove(QMouseEvent *me) { 

}

void ScanToolsBoxColorDialogImpl::keyPressEvent(QKeyEvent *ke) {
	ke->ignore();
}

void ScanToolsBoxColorDialogImpl::keyReleaseEvent(QKeyEvent *ke) {
	ke->ignore();
}

void ScanToolsBoxColorDialogImpl::passKeyPressEvent(QKeyEvent   *ke) {
	ke->ignore();
}

void ScanToolsBoxColorDialogImpl::passKeyReleaseEvent(QKeyEvent   *ke) {
	ke->ignore();
	
}

void ScanToolsBoxColorDialogImpl::setQCursor(QCursor *new_cursor) {
	
}

void ScanToolsBoxColorDialogImpl::handleApplyButton() {
	float color[4];
	float sel_color[4];
	bool success = true	;
	bool s = false;
	
	color[0] = RedEdit->text().toFloat(&s);
	success = success && s;
	color[1] = GreenEdit->text().toFloat(&s);
	success = success && s;
	color[2] = BlueEdit->text().toFloat(&s);
	success = success && s;
	color[3] = AlphaEdit->text().toFloat(&s);
	success = success && s;

	sel_color[0] = SelectionRedEdit->text().toFloat(&s);
	success = success && s;
	sel_color[1] = SelectionGreenEdit->text().toFloat(&s);
	success = success && s;
	sel_color[2] = SelectionBlueEdit->text().toFloat(&s);
	success = success && s;
	sel_color[3] = 1;

	if (success)
		the_box_->setColors(color,sel_color);	
	else {
		ScanToolsConfigurationDialogImpl *w = ScanToolsConfigurationDialogImpl::getInstance();
		w->setWrongArgumentsMessage();
		w->show();
	}
}

void ScanToolsBoxColorDialogImpl::setBoxSelector(BoxSelector *box) {
	the_box_ = box;
}


// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
