// Title:   Selection.h
// Created: Thu Sep 25 15:15:23 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __SELECTION_H
#define __SELECTION_H

#include "../../../../Core/Scene/src/Scene.h"

#include "Selector.h" 

#if QT_VERSION >= 0x030000
# include <qptrvector.h>
# ifndef QVector
#  define QVector QPtrVector
# endif
# ifndef QVectorIterator
#  define QVectorIterator QPtrVectorIterator
# endif
#else
# include <qvector.h>
#endif

class Selection {
	friend class Selector;

	Selector *_selector;

	SurfelInterface::Flag _flag;
	Object *_from_object;
	
	int *_selected_surfels;
	unsigned int _selection_count;

	bool _iteration_active;
	unsigned int _current_iteration_index;

	Selection() {};
	void buildSelectionArray(void);

public:
	Selection(SurfelInterface::Flag flag, int selection_count, Object *current_object);
	~Selection();

	int getNOfSurfels(void);

	/* 
	** those two operators should only be accessed if NO ITERATION IS ACTIVE !!!
	*/
	inline SurfelInterface* operator[](int index);
	//inline Vector3D& operator()(int index);
	inline bool isIterationActive(void);

	/*
	** Two iteration schemes
	*/
	SurfelInterface* getFirstSurfel();
	SurfelInterface* getNextSurfel();

	//Vector3D& getFirstPosition();
	//Vector3D& getNextPosition();
};

#endif
// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
